@@ -24,8 +24,9 @@
  -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
-/*
- *  readfile.c:  reads image on file into memory
+/*!
+ * \file readfile.c:  reads image on file into memory
+ * <pre>
  *
  *      Top-level functions for reading images from file
  *           PIXA      *pixaReadFiles()
@@ -48,6 +49,9 @@
  *           PIX       *pixReadMem()
  *           l_int32    pixReadHeaderMem()
  *
+ *      Output image file information
+ *           void       writeImageFileInfo()
+ *
  *      Test function for I/O with different formats
  *           l_int32    ioFormatTest()
  *
@@ -64,26 +68,32 @@
  *          webp
  *          jp2 (jpeg 2000)
  *  (3) Other file types will get an "unknown format" error.
+ * </pre>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include <string.h>
 #include "allheaders.h"
 
     /* Output files for ioFormatTest(). */
-static const char *FILE_BMP  =  "/tmp/lept/format.bmp";
-static const char *FILE_PNG  =  "/tmp/lept/format.png";
-static const char *FILE_PNM  =  "/tmp/lept/format.pnm";
-static const char *FILE_G3   =  "/tmp/lept/format_g3.tif";
-static const char *FILE_G4   =  "/tmp/lept/format_g4.tif";
-static const char *FILE_RLE  =  "/tmp/lept/format_rle.tif";
-static const char *FILE_PB   =  "/tmp/lept/format_packbits.tif";
-static const char *FILE_LZW  =  "/tmp/lept/format_lzw.tif";
-static const char *FILE_ZIP  =  "/tmp/lept/format_zip.tif";
-static const char *FILE_TIFF =  "/tmp/lept/format.tif";
-static const char *FILE_JPG  =  "/tmp/lept/format.jpg";
-static const char *FILE_GIF  =  "/tmp/lept/format.gif";
-static const char *FILE_WEBP =  "/tmp/lept/format.webp";
-static const char *FILE_JP2K =  "/tmp/lept/format.jp2";
+static const char *FILE_BMP  =  "/tmp/lept/format/file.bmp";
+static const char *FILE_PNG  =  "/tmp/lept/format/file.png";
+static const char *FILE_PNM  =  "/tmp/lept/format/file.pnm";
+static const char *FILE_G3   =  "/tmp/lept/format/file_g3.tif";
+static const char *FILE_G4   =  "/tmp/lept/format/file_g4.tif";
+static const char *FILE_RLE  =  "/tmp/lept/format/file_rle.tif";
+static const char *FILE_PB   =  "/tmp/lept/format/file_packbits.tif";
+static const char *FILE_LZW  =  "/tmp/lept/format/file_lzw.tif";
+static const char *FILE_ZIP  =  "/tmp/lept/format/file_zip.tif";
+static const char *FILE_TIFF_JPEG =  "/tmp/lept/format/file_jpeg.tif";
+static const char *FILE_TIFF =  "/tmp/lept/format/file.tif";
+static const char *FILE_JPG  =  "/tmp/lept/format/file.jpg";
+static const char *FILE_GIF  =  "/tmp/lept/format/file.gif";
+static const char *FILE_WEBP =  "/tmp/lept/format/file.webp";
+static const char *FILE_JP2K =  "/tmp/lept/format/file.jp2";
 
 static const unsigned char JP2K_CODESTREAM[4] = { 0xff, 0x4f, 0xff, 0x51 };
 static const unsigned char JP2K_IMAGE_DATA[12] = { 0x00, 0x00, 0x00, 0x0C,
@@ -95,18 +105,20 @@ static const unsigned char JP2K_IMAGE_DATA[12] = { 0x00, 0x00, 0x00, 0x0C,
  *          Top-level functions for reading images from file           *
  *---------------------------------------------------------------------*/
 /*!
- *  pixaReadFiles()
+ * \brief   pixaReadFiles()
  *
- *      Input:  dirname
- *              substr (<optional> substring filter on filenames; can be null)
- *      Return: pixa, or null on error
+ * \param[in]    dirname
+ * \param[in]    substr   [optional] substring filter on filenames; can be null
+ * \return  pixa, or NULL on error
  *
- *  Notes:
- *      (1) @dirname is the full path for the directory.
- *      (2) @substr is the part of the file name (excluding
+ * <pre>
+ * Notes:
+ *      (1) %dirname is the full path for the directory.
+ *      (2) %substr is the part of the file name (excluding
  *          the directory) that is to be matched.  All matching
  *          filenames are read into the Pixa.  If substr is NULL,
  *          all filenames are read into the Pixa.
+ * </pre>
  */
 PIXA *
 pixaReadFiles(const char  *dirname,
@@ -130,10 +142,10 @@ SARRAY  *sa;
 
 
 /*!
- *  pixaReadFilesSA()
+ * \brief   pixaReadFilesSA()
  *
- *      Input:  sarray (full pathnames for all files)
- *      Return: pixa, or null on error
+ * \param[in]    sa     full pathnames for all files
+ * \return  pixa, or NULL on error
  */
 PIXA *
 pixaReadFilesSA(SARRAY  *sa)
@@ -164,13 +176,15 @@ PIXA    *pixa;
 
 
 /*!
- *  pixRead()
+ * \brief   pixRead()
  *
- *      Input:  filename (with full pathname or in local directory)
- *      Return: pix if OK; null on error
+ * \param[in]    filename    with full pathname or in local directory
+ * \return  pix if OK; NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) See at top of file for supported formats.
+ * </pre>
  */
 PIX *
 pixRead(const char  *filename)
@@ -187,26 +201,27 @@ PIX   *pix;
         L_ERROR("image file not found: %s\n", procName, filename);
         return NULL;
     }
-    if ((pix = pixReadStream(fp, 0)) == NULL) {
-        fclose(fp);
-        return (PIX *)ERROR_PTR("pix not read", procName, NULL);
-    }
+    pix = pixReadStream(fp, 0);
     fclose(fp);
-
+    if (!pix)
+        return (PIX *)ERROR_PTR("pix not read", procName, NULL);
     return pix;
 }
 
 
 /*!
- *  pixReadWithHint()
+ * \brief   pixReadWithHint()
  *
- *      Input:  filename (with full pathname or in local directory)
- *              hint (bitwise OR of L_HINT_* values for jpeg; use 0 for no hint)
- *      Return: pix if OK; null on error
+ * \param[in]    filename    with full pathname or in local directory
+ * \param[in]    hint        bitwise OR of L_HINT_* values for jpeg;
+ *                           use 0 for no hint
+ * \return  pix if OK; NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) The hint is not binding, but may be used to optimize jpeg decoding.
  *          Use 0 for no hinting.
+ * </pre>
  */
 PIX *
 pixReadWithHint(const char  *filename,
@@ -232,19 +247,20 @@ PIX   *pix;
 
 
 /*!
- *  pixReadIndexed()
+ * \brief   pixReadIndexed()
  *
- *      Input:  sarray (of full pathnames)
- *              index (into pathname array)
- *      Return: pix if OK; null if not found
+ * \param[in]    sa      string array of full pathnames
+ * \param[in]    index   into pathname array
+ * \return  pix if OK; null if not found
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This function is useful for selecting image files from a
- *          directory, where the integer @index is embedded into
+ *          directory, where the integer %index is embedded into
  *          the file name.
  *      (2) This is typically done by generating the sarray using
- *          getNumberedPathnamesInDirectory(), so that the @index
- *          pathname would have the number @index in it.  The size
+ *          getNumberedPathnamesInDirectory(), so that the %index
+ *          pathname would have the number %index in it.  The size
  *          of the sarray should be the largest number (plus 1) appearing
  *          in the file names, respecting the constraints in the
  *          call to getNumberedPathnamesInDirectory().
@@ -256,6 +272,7 @@ PIX   *pix;
  *          Therefore, the caller must verify that a pix is returned.
  *      (4) See convertSegmentedPagesToPS() in src/psio1.c for an
  *          example of usage.
+ * </pre>
  */
 PIX *
 pixReadIndexed(SARRAY  *sa,
@@ -287,22 +304,25 @@ PIX     *pix;
 
 
 /*!
- *  pixReadStream()
+ * \brief   pixReadStream()
  *
- *      Input:  fp (file stream)
- *              hint (bitwise OR of L_HINT_* values for jpeg; use 0 for no hint)
- *      Return: pix if OK; null on error
+ * \param[in]    fp      file stream
+ * \param[in]    hint    bitwise OR of L_HINT_* values for jpeg; 0 for no hint
+ * \return  pix if OK; NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) The hint only applies to jpeg.
+ * </pre>
  */
 PIX *
 pixReadStream(FILE    *fp,
               l_int32  hint)
 {
-l_int32   format, ret;
+l_int32   format, ret, valid;
 l_uint8  *comment;
 PIX      *pix;
+PIXCMAP  *cmap;
 
     PROCNAME("pixReadStream");
 
@@ -324,7 +344,7 @@ PIX      *pix;
         ret = fgetJpegComment(fp, &comment);
         if (!ret && comment)
             pixSetText(pix, (char *)comment);
-        FREE(comment);
+        LEPT_FREE(comment);
         break;
 
     case IFF_PNG:
@@ -339,6 +359,7 @@ PIX      *pix;
     case IFF_TIFF_G4:
     case IFF_TIFF_LZW:
     case IFF_TIFF_ZIP:
+    case IFF_TIFF_JPEG:
         if ((pix = pixReadStreamTiff(fp, 0)) == NULL)  /* page 0 by default */
             return (PIX *)ERROR_PTR("tiff: no pix returned", procName, NULL);
         break;
@@ -363,6 +384,14 @@ PIX      *pix;
             return (PIX *)ERROR_PTR("webp: no pix returned", procName, NULL);
         break;
 
+    case IFF_PS:
+        L_ERROR("PostScript reading is not supported\n", procName);
+        return NULL;
+
+    case IFF_LPDF:
+        L_ERROR("Pdf reading is not supported\n", procName);
+        return NULL;
+
     case IFF_SPIX:
         if ((pix = pixReadStreamSpix(fp)) == NULL)
             return (PIX *)ERROR_PTR("spix: no pix returned", procName, NULL);
@@ -374,8 +403,16 @@ PIX      *pix;
         break;
     }
 
-    if (pix)
+    if (pix) {
         pixSetInputFormat(pix, format);
+        if ((cmap = pixGetColormap(pix))) {
+            pixcmapIsValid(cmap, &valid);
+            if (!valid) {
+                pixDestroy(&pix);
+                return (PIX *)ERROR_PTR("invalid colormap", procName, NULL);
+            }
+        }
+    }
     return pix;
 }
 
@@ -385,22 +422,24 @@ PIX      *pix;
  *                     Read header information from file               *
  *---------------------------------------------------------------------*/
 /*!
- *  pixReadHeader()
+ * \brief   pixReadHeader()
  *
- *      Input:  filename (with full pathname or in local directory)
- *              &format (<optional return> file format)
- *              &w, &h (<optional returns> width and height)
- *              &bps <optional return> bits/sample
- *              &spp <optional return> samples/pixel (1, 3 or 4)
- *              &iscmap (<optional return> 1 if cmap exists; 0 otherwise)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    filename    with full pathname or in local directory
+ * \param[out]   pformat     [optional] file format
+ * \param[out]   pw, ph      [optional] width and height
+ * \param[out]   pbps        [optional] bits/sample
+ * \param[out]   pspp        [optional] samples/pixel 1, 3 or 4
+ * \param[out]   piscmap     [optional] 1 if cmap exists; 0 otherwise
+ * \return  0 if OK, 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This reads the actual headers for jpeg, png, tiff and pnm.
  *          For bmp and gif, we cheat and read the entire file into a pix,
  *          from which we extract the "header" information.
+ * </pre>
  */
-l_int32
+l_ok
 pixReadHeader(const char  *filename,
               l_int32     *pformat,
               l_int32     *pw,
@@ -464,6 +503,7 @@ PIX     *pix;
     case IFF_TIFF_G4:
     case IFF_TIFF_LZW:
     case IFF_TIFF_ZIP:
+    case IFF_TIFF_JPEG:
             /* Reading page 0 by default; possibly redefine format */
         ret = readHeaderTiff(filename, 0, &w, &h, &bps, &spp, NULL, &iscmap,
                              &format);
@@ -497,6 +537,14 @@ PIX     *pix;
         bps = 8;
         break;
 
+    case IFF_PS:
+        if (pformat) *pformat = format;
+        return ERROR_INT("PostScript reading is not supported\n", procName, 1);
+
+    case IFF_LPDF:
+        if (pformat) *pformat = format;
+        return ERROR_INT("Pdf reading is not supported\n", procName, 1);
+
     case IFF_SPIX:
         ret = readHeaderSpix(filename, &w, &h, &bps, &spp, &iscmap);
         if (ret)
@@ -523,13 +571,13 @@ PIX     *pix;
  *                            Format finders                           *
  *---------------------------------------------------------------------*/
 /*!
- *  findFileFormat()
+ * \brief   findFileFormat()
  *
- *      Input:  filename
- *              &format (<return>)
- *      Return: 0 if OK, 1 on error or if format is not recognized
+ * \param[in]    filename
+ * \param[out]   pformat    found format
+ * \return  0 if OK, 1 on error or if format is not recognized
  */
-l_int32
+l_ok
 findFileFormat(const char  *filename,
                l_int32     *pformat)
 {
@@ -553,16 +601,18 @@ FILE    *fp;
 
 
 /*!
- *  findFileFormatStream()
+ * \brief   findFileFormatStream()
  *
- *      Input:  fp (file stream)
- *              &format (<return>)
- *      Return: 0 if OK, 1 on error or if format is not recognized
+ * \param[in]    fp        file stream
+ * \param[out]   pformat   found format
+ * \return  0 if OK, 1 on error or if format is not recognized
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) Important: Side effect -- this resets fp to BOF.
+ * </pre>
  */
-l_int32
+l_ok
 findFileFormatStream(FILE     *fp,
                      l_int32  *pformat)
 {
@@ -581,7 +631,7 @@ l_int32  format;
     if (fnbytesInFile(fp) < 12)
         return ERROR_INT("truncated file", procName, 1);
 
-    if (fread((char *)&firstbytes, 1, 12, fp) != 12)
+    if (fread(&firstbytes, 1, 12, fp) != 12)
         return ERROR_INT("failed to read first 12 bytes of file", procName, 1);
     rewind(fp);
 
@@ -599,19 +649,21 @@ l_int32  format;
 
 
 /*!
- *  findFileFormatBuffer()
+ * \brief   findFileFormatBuffer()
  *
- *      Input:  byte buffer (at least 12 bytes in size; we can't check)
- *              &format (<return>)
- *      Return: 0 if OK, 1 on error or if format is not recognized
+ * \param[in]    buf       byte buffer at least 12 bytes in size; we can't check
+ * \param[out]   pformat   found format
+ * \return  0 if OK, 1 on error or if format is not recognized
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This determines the file format from the first 12 bytes in
  *          the compressed data stream, which are stored in memory.
  *      (2) For tiff files, this returns IFF_TIFF.  The specific tiff
  *          compression is then determined using findTiffCompression().
+ * </pre>
  */
-l_int32
+l_ok
 findFileFormatBuffer(const l_uint8  *buf,
                      l_int32        *pformat)
 {
@@ -641,7 +693,7 @@ l_uint16  twobytepw;
 
         /* Check for the p*m 2-byte header ids */
     if ((buf[0] == 'P' && buf[1] == '4') || /* newer packed */
-        (buf[0] == 'P' && buf[1] == '1')) {  /* old format */
+        (buf[0] == 'P' && buf[1] == '1')) {  /* old ASCII format */
         *pformat = IFF_PNM;
         return 0;
     }
@@ -658,6 +710,11 @@ l_uint16  twobytepw;
         return 0;
     }
 
+    if (buf[0] == 'P' && buf[1] == '7') {  /* new arbitrary (PAM) */
+        *pformat = IFF_PNM;
+        return 0;
+    }
+
         /*  Consider the first 11 bytes of the standard JFIF JPEG header:
          *    - The first two bytes are the most important:  0xffd8.
          *    - The next two bytes are the jfif marker: 0xffe0.
@@ -690,8 +747,8 @@ l_uint16  twobytepw;
     }
 
         /* Check for both types of jp2k file */
-    if (strncmp((const char *)buf, (char *)JP2K_CODESTREAM, 4) == 0 ||
-        strncmp((const char *)buf, (char *)JP2K_IMAGE_DATA, 12) == 0) {
+    if (memcmp(buf, JP2K_CODESTREAM, 4) == 0 ||
+        memcmp(buf, JP2K_IMAGE_DATA, 12) == 0) {
         *pformat = IFF_JP2;
         return 0;
     }
@@ -703,6 +760,21 @@ l_uint16  twobytepw;
         return 0;
     }
 
+        /* Check for ps */
+    if (buf[0] == '%' && buf[1] == '!' && buf[2] == 'P' && buf[3] == 'S' &&
+        buf[4] == '-' && buf[5] == 'A' && buf[6] == 'd' && buf[7] == 'o' &&
+        buf[8] == 'b' && buf[9] == 'e') {
+        *pformat = IFF_PS;
+        return 0;
+    }
+
+        /* Check for pdf */
+    if (buf[0] == '%' && buf[1] == 'P' && buf[2] == 'D' && buf[3] == 'F' &&
+        buf[4] == '-' && buf[5] == '1') {
+        *pformat = IFF_LPDF;
+        return 0;
+    }
+
         /* Check for "spix" serialized pix */
     if (buf[0] == 's' && buf[1] == 'p' && buf[2] == 'i' && buf[3] == 'x') {
         *pformat = IFF_SPIX;
@@ -715,10 +787,10 @@ l_uint16  twobytepw;
 
 
 /*!
- *  fileFormatIsTiff()
+ * \brief   fileFormatIsTiff()
  *
- *      Input:  fp (file stream)
- *      Return: 1 if file is tiff; 0 otherwise or on error
+ * \param[in]    fp    file stream
+ * \return  1 if file is tiff; 0 otherwise or on error
  */
 l_int32
 fileFormatIsTiff(FILE  *fp)
@@ -734,7 +806,7 @@ l_int32  format;
     if (format == IFF_TIFF || format == IFF_TIFF_PACKBITS ||
         format == IFF_TIFF_RLE || format == IFF_TIFF_G3 ||
         format == IFF_TIFF_G4 || format == IFF_TIFF_LZW ||
-        format == IFF_TIFF_ZIP)
+        format == IFF_TIFF_ZIP || format == IFF_TIFF_JPEG)
         return 1;
     else
         return 0;
@@ -745,29 +817,32 @@ l_int32  format;
  *                            Read from memory                         *
  *---------------------------------------------------------------------*/
 /*!
- *  pixReadMem()
+ * \brief   pixReadMem()
  *
- *      Input:  data (const; encoded)
- *              datasize (size of data)
- *      Return: pix, or null on error
+ * \param[in]    data    const; encoded
+ * \param[in]    size    size of data
+ * \return  pix, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This is a variation of pixReadStream(), where the data is read
  *          from a memory buffer rather than a file.
  *      (2) On windows, this only reads tiff formatted files directly from
- *          memory.  For other formats, it write to a temp file and
- *          decompress from file.
+ *          memory.  For other formats, it writes to a temp file and
+ *          decompresses from file.
  *      (3) findFileFormatBuffer() requires up to 12 bytes to decide on
  *          the format.  That determines the constraint here.  But in
  *          fact the data must contain the entire compressed string for
  *          the image.
+ * </pre>
  */
 PIX *
 pixReadMem(const l_uint8  *data,
            size_t          size)
 {
-l_int32  format;
-PIX     *pix;
+l_int32   format, valid;
+PIX      *pix;
+PIXCMAP  *cmap;
 
     PROCNAME("pixReadMem");
 
@@ -827,6 +902,14 @@ PIX     *pix;
             return (PIX *)ERROR_PTR("webp: no pix returned", procName, NULL);
         break;
 
+    case IFF_PS:
+        L_ERROR("PostScript reading is not supported\n", procName);
+        return NULL;
+
+    case IFF_LPDF:
+        L_ERROR("Pdf reading is not supported\n", procName);
+        return NULL;
+
     case IFF_SPIX:
         if ((pix = pixReadMemSpix(data, size)) == NULL)
             return (PIX *)ERROR_PTR("spix: no pix returned", procName, NULL);
@@ -839,30 +922,39 @@ PIX     *pix;
     }
 
         /* Set the input format.  For tiff reading from memory we lose
-         * the actual input format; for 1 bpp, default to G4.  */
+         * the actual input format; for 1 bpp, default to G4.  Also
+         * verify that the colormap is valid.  */
     if (pix) {
         if (format == IFF_TIFF && pixGetDepth(pix) == 1)
             format = IFF_TIFF_G4;
         pixSetInputFormat(pix, format);
+        if ((cmap = pixGetColormap(pix))) {
+            pixcmapIsValid(cmap, &valid);
+            if (!valid) {
+                pixDestroy(&pix);
+                return (PIX *)ERROR_PTR("invalid colormap", procName, NULL);
+            }
+        }
+        pixSetPadBits(pix, 0);
     }
-
     return pix;
 }
 
 
 /*!
- *  pixReadHeaderMem()
+ * \brief   pixReadHeaderMem()
  *
- *      Input:  data (const; encoded)
- *              datasize (size of data)
- *              &format (<optional returns> image format)
- *              &w, &h (<optional returns> width and height)
- *              &bps <optional return> bits/sample
- *              &spp <optional return> samples/pixel (1, 3 or 4)
- *              &iscmap (<optional return> 1 if cmap exists; 0 otherwise)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    data       const; encoded
+ * \param[in]    size       size of data
+ * \param[out]   pformat    [optional] image format
+ * \param[out]   pw, ph     [optional] width and height
+ * \param[out]   pbps       [optional] bits/sample
+ * \param[out]   pspp       [optional] samples/pixel 1, 3 or 4
+ * \param[out]   piscmap    [optional] 1 if cmap exists; 0 otherwise
+ * \return  0 if OK, 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This reads the actual headers for jpeg, png, tiff, jp2k and pnm.
  *          For bmp and gif, we cheat and read all the data into a pix,
  *          from which we extract the "header" information.
@@ -870,10 +962,11 @@ PIX     *pix;
  *          png, it requires less than 30 bytes, but for jpeg it can
  *          require most of the compressed file.  In practice, the data
  *          is typically the entire compressed file in memory.
- *      (3) findFileFormatBuffer() requires up to 8 bytes to decide on
+ *      (3) findFileFormatBuffer() requires up to 12 bytes to decide on
  *          the format, which we require.
+ * </pre>
  */
-l_int32
+l_ok
 pixReadHeaderMem(const l_uint8  *data,
                  size_t          size,
                  l_int32        *pformat,
@@ -898,8 +991,8 @@ PIX     *pix;
     iscmap = 0;  /* init to false */
     if (!data)
         return ERROR_INT("data not defined", procName, 1);
-    if (size < 8)
-        return ERROR_INT("size < 8", procName, 1);
+    if (size < 12)
+        return ERROR_INT("size < 12", procName, 1);
 
     findFileFormatBuffer(data, &format);
 
@@ -934,6 +1027,7 @@ PIX     *pix;
     case IFF_TIFF_G4:
     case IFF_TIFF_LZW:
     case IFF_TIFF_ZIP:
+    case IFF_TIFF_JPEG:
             /* Reading page 0 by default; possibly redefine format */
         ret = readHeaderMemTiff(data, size, 0, &w, &h, &bps, &spp,
                                 NULL, &iscmap, &format);
@@ -966,6 +1060,14 @@ PIX     *pix;
         ret = readHeaderMemWebP(data, size, &w, &h, &spp);
         break;
 
+    case IFF_PS:
+        if (pformat) *pformat = format;
+        return ERROR_INT("PostScript reading is not supported\n", procName, 1);
+
+    case IFF_LPDF:
+        if (pformat) *pformat = format;
+        return ERROR_INT("Pdf reading is not supported\n", procName, 1);
+
     case IFF_SPIX:
         ret = sreadHeaderSpix((l_uint32 *)data, &w, &h, &bps,
                                &spp, &iscmap);
@@ -989,19 +1091,182 @@ PIX     *pix;
 
 
 /*---------------------------------------------------------------------*
- *             Test function for I/O with different formats            *
+ *                    Output image file information                    *
  *---------------------------------------------------------------------*/
-#ifdef HAVE_CONFIG_H
-#include "config_auto.h"
-#endif  /* HAVE_CONFIG_H */
+extern const char *ImageFileFormatExtensions[];
 
 /*!
- *  ioFormatTest()
+ * \brief   writeImageFileInfo()
  *
- *      Input:  filename (input file)
- *      Return: 0 if OK; 1 on error or if the test fails
+ * \param[in]    filename    input file
+ * \param[in]    fpout       output file stream
+ * \param[in]    headeronly  1 to read only the header; 0 to read both
+ *                           the header and the input file
+ * \return  0 if OK; 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
+ *      (1) If headeronly == 0 and the image has spp == 4,this will
+ *          also call pixDisplayLayersRGBA() to display the image
+ *          in three views.
+ *      (2) This is a debug function that changes the value of
+ *          var_PNG_STRIP_16_TO_8 to 1 (the default).
+ * </pre>
+ */
+l_ok
+writeImageFileInfo(const char  *filename,
+                   FILE        *fpout,
+                   l_int32      headeronly)
+{
+char     *text;
+l_int32   w, h, d, wpl, count, npages, color;
+l_int32   format, bps, spp, iscmap, xres, yres, transparency;
+FILE     *fpin;
+PIX      *pix, *pixt;
+PIXCMAP  *cmap;
+
+    PROCNAME("writeImageFileInfo");
+
+    if (!filename)
+        return ERROR_INT("filename not defined", procName, 1);
+    if (!fpout)
+        return ERROR_INT("stream not defined", procName, 1);
+
+        /* Read the header */
+    if (pixReadHeader(filename, &format, &w, &h, &bps, &spp, &iscmap)) {
+        L_ERROR("failure to read header of %s\n", procName, filename);
+        return 1;
+    }
+    fprintf(fpout, "===============================================\n"
+                    "Reading the header:\n");
+    fprintf(fpout, "  input image format type: %s\n",
+            ImageFileFormatExtensions[format]);
+    fprintf(fpout, "  w = %d, h = %d, bps = %d, spp = %d, iscmap = %d\n",
+            w, h, bps, spp, iscmap);
+
+    findFileFormat(filename, &format);
+    if (format == IFF_JP2) {
+        fpin = lept_fopen(filename, "rb");
+        fgetJp2kResolution(fpin, &xres, &yres);
+        fclose(fpin);
+        fprintf(fpout, "  xres = %d, yres = %d\n", xres, yres);
+    } else if (format == IFF_PNG) {
+        fpin = lept_fopen(filename, "rb");
+        fgetPngResolution(fpin, &xres, &yres);
+        fclose(fpin);
+        fprintf(fpout, "  xres = %d, yres = %d\n", xres, yres);
+        if (iscmap) {
+            fpin = lept_fopen(filename, "rb");
+            fgetPngColormapInfo(fpin, &cmap, &transparency);
+            fclose(fpin);
+            if (transparency)
+                fprintf(fpout, "  colormap has transparency\n");
+            else
+                fprintf(fpout, "  colormap does not have transparency\n");
+            pixcmapWriteStream(fpout, cmap);
+            pixcmapDestroy(&cmap);
+        }
+    } else if (format == IFF_JFIF_JPEG) {
+        fpin = lept_fopen(filename, "rb");
+        fgetJpegResolution(fpin, &xres, &yres);
+        fclose(fpin);
+        fprintf(fpout, "  xres = %d, yres = %d\n", xres, yres);
+    }
+
+    if (headeronly)
+        return 0;
+
+        /* Read the full image.  Note that when we read an image that
+         * has transparency in a colormap, we convert it to RGBA. */
+    fprintf(fpout, "===============================================\n"
+                    "Reading the full image:\n");
+
+        /* Preserve 16 bpp if the format is png */
+    if (format == IFF_PNG && bps == 16)
+        l_pngSetReadStrip16To8(0);
+
+    if ((pix = pixRead(filename)) == NULL) {
+        L_ERROR("failure to read full image of %s\n", procName, filename);
+        return 1;
+    }
+
+    format = pixGetInputFormat(pix);
+    pixGetDimensions(pix, &w, &h, &d);
+    wpl = pixGetWpl(pix);
+    spp = pixGetSpp(pix);
+    fprintf(fpout, "  input image format type: %s\n",
+            ImageFileFormatExtensions[format]);
+    fprintf(fpout, "  w = %d, h = %d, d = %d, spp = %d, wpl = %d\n",
+            w, h, d, spp, wpl);
+    fprintf(fpout, "  xres = %d, yres = %d\n",
+            pixGetXRes(pix), pixGetYRes(pix));
+
+    text = pixGetText(pix);
+    if (text)  /*  not null */
+        fprintf(fpout, "  text: %s\n", text);
+
+    cmap = pixGetColormap(pix);
+    if (cmap) {
+        pixcmapHasColor(cmap, &color);
+        if (color)
+            fprintf(fpout, "  colormap exists and has color values:");
+        else
+            fprintf(fpout, "  colormap exists and has only gray values:");
+        pixcmapWriteStream(fpout, pixGetColormap(pix));
+    }
+    else
+        fprintf(fpout, "  colormap does not exist\n");
+
+    if (format == IFF_TIFF || format == IFF_TIFF_G4 ||
+        format == IFF_TIFF_G3 || format == IFF_TIFF_PACKBITS) {
+        fprintf(fpout, "  Tiff header information:\n");
+        fpin = lept_fopen(filename, "rb");
+        tiffGetCount(fpin, &npages);
+        lept_fclose(fpin);
+        if (npages == 1)
+            fprintf(fpout, "    One page in file\n");
+        else
+            fprintf(fpout, "    %d pages in file\n", npages);
+        fprintTiffInfo(fpout, filename);
+    }
+
+    if (d == 1) {
+        pixCountPixels(pix, &count, NULL);
+        pixGetDimensions(pix, &w, &h, NULL);
+        fprintf(fpout, "  1 bpp: foreground pixel fraction ON/Total = %g\n",
+                (l_float32)count / (l_float32)(w * h));
+    }
+    fprintf(fpout, "===============================================\n");
+
+        /* If there is an alpha component, visualize it.  Note that when
+         * alpha == 0, the rgb layer is transparent.  We visualize the
+         * result when a white background is visible through the
+         * transparency layer. */
+    if (pixGetSpp(pix) == 4) {
+        pixt = pixDisplayLayersRGBA(pix, 0xffffff00, 600.0);
+        pixDisplay(pixt, 100, 100);
+        pixDestroy(&pixt);
+    }
+
+    if (format == IFF_PNG && bps == 16)
+        l_pngSetReadStrip16To8(1);  /* return to default if format is png */
+
+    pixDestroy(&pix);
+    return 0;
+}
+
+
+/*---------------------------------------------------------------------*
+ *             Test function for I/O with different formats            *
+ *---------------------------------------------------------------------*/
+/*!
+ * \brief   ioFormatTest()
+ *
+ * \param[in]    filename    input image file
+ * \return  0 if OK; 1 on error or if the test fails
+ *
+ * <pre>
+ * Notes:
  *      (1) This writes and reads a set of output files losslessly
  *          in different formats to /tmp/format/, and tests that the
  *          result before and after is unchanged.
@@ -1014,8 +1279,9 @@ PIX     *pix;
  *          16 bpp png, although this can be turned on in pngio.c.
  *      (4) This silently skips png or tiff testing if HAVE_LIBPNG
  *          or HAVE_LIBTIFF are 0, respectively.
+ * </pre>
  */
-l_int32
+l_ok
 ioFormatTest(const char  *filename)
 {
 l_int32    w, h, d, depth, equal, problems;
@@ -1042,7 +1308,7 @@ PIXCMAP   *cmap;
     }
     pixDestroy(&pix1);
 
-    lept_mkdir("lept");
+    lept_mkdir("lept/format");
 
         /* Note that the reader automatically removes colormaps
          * from 1 bpp BMP images, but not from 8 bpp BMP images.
@@ -1160,6 +1426,37 @@ PIXCMAP   *cmap;
     }
     pixDestroy(&pix1);
 
+        /* tiff jpeg encoding works for grayscale and rgb */
+    if (d == 8 || d == 32) {
+        PIX  *pixc1;
+        L_INFO("write/read jpeg compressed tiff\n", procName);
+        if (d == 8 && pixGetColormap(pixc)) {
+            pixc1 = pixRemoveColormap(pixc, REMOVE_CMAP_BASED_ON_SRC);
+            pixWrite(FILE_TIFF_JPEG, pixc1, IFF_TIFF_JPEG);
+            if ((pix1 = pixRead(FILE_TIFF_JPEG)) == NULL) {
+                L_INFO(" did not read FILE_TIFF_JPEG\n", procName);
+                problems = TRUE;
+            }
+            pixDestroy(&pixc1);
+        } else {
+            pixWrite(FILE_TIFF_JPEG, pixc, IFF_TIFF_JPEG);
+            pix1 = pixRead(FILE_TIFF_JPEG);
+            if (d == 8) {
+                pixCompareGray(pix1, pixc, L_COMPARE_ABS_DIFF, 0, NULL, &diff,
+                               NULL, NULL);
+            } else {
+                pixCompareRGB(pix1, pixc, L_COMPARE_ABS_DIFF, 0, NULL, &diff,
+                              NULL, NULL);
+            }
+            if (diff > 8.0) {
+                L_INFO("   **** bad tiff jpeg compressed image: "
+                       "d = %d, diff = %5.2f ****\n", procName, d, diff);
+                problems = TRUE;
+            }
+        }
+        pixDestroy(&pix1);
+    }
+
         /* tiff g4, g3, rle and packbits work for 1 bpp */
     if (d == 1) {
         L_INFO("write/read g4 compressed tiff\n", procName);
@@ -1311,7 +1608,7 @@ PIXCMAP   *cmap;
         pixCompareRGB(pix1, pix2, L_COMPARE_ABS_DIFF, 0, NULL, &diff,
                       NULL, NULL);
     }
-    fprintf(stderr, "diff = %7.3f\n", diff);
+    lept_stderr("diff = %7.3f\n", diff);
     if (diff > 7.0) {
         L_INFO("   **** bad jp2k image: d = %d, diff = %5.2f ****\n",
                procName, depth, diff);
@@ -1328,4 +1625,3 @@ PIXCMAP   *cmap;
     pixDestroy(&pixs);
     return problems;
 }
-
